from collections.abc import Callable
import warnings

import gymnasium as gym
import numpy as np

from pydrake.common import RandomGenerator
from pydrake.systems.analysis import Simulator, SimulatorStatus
from pydrake.systems.framework import (
    Context,
    InputPort,
    InputPortIndex,
    OutputPortIndex,
    PortDataType,
    System,
)
from pydrake.systems.sensors import ImageRgba8U


def _reached_termination(status: SimulatorStatus):
    """Returns true iff status is "reached termination condition"."""
    ReturnReason = SimulatorStatus.ReturnReason
    return status.reason() == ReturnReason.kReachedTerminationCondition


class DrakeGymEnv(gym.Env):
    """
    DrakeGymEnv provides a gym.Env interface for a Drake System (often a
    Diagram) using a Simulator.
    """

    def __init__(
        self,
        simulator: Simulator | Callable[[RandomGenerator], Simulator],
        time_step: float,
        action_space: gym.spaces.Space,
        observation_space: gym.spaces.Space,
        reward: Callable[[System, Context], float] | OutputPortIndex | str,
        action_port_id: InputPort | InputPortIndex | str = None,
        observation_port_id: OutputPortIndex | str = None,
        render_rgb_port_id: OutputPortIndex | str = None,
        render_mode: str = "human",
        reset_handler: Callable[[Simulator, Context], None] | None = None,
        info_handler: Callable[[Simulator], dict] | None = None,
        hardware: bool = False,
    ):
        """
        Args:
            simulator: Either a ``drake.systems.analysis.Simulator``, or
                a function that produces a (randomized) Simulator.
            time_step: Each call to ``step()`` will advance the simulator by
                ``time_step`` seconds.
            action_space: Defines the ``gym.spaces.Space`` for the actions.  If
                the action port is vector-valued, then passing ``None``
                defaults to a ``gym.spaces.Box`` of the correct dimension with
                bounds at negative and positive infinity.  Note: Stable
                Baselines 3 strongly encourages normalizing the
                ``action_space`` to [-1, 1].
            observation_space: Defines the ``gym.spaces.Space`` for the
                observations.  If the observation port is vector-valued, then
                passing ``None`` defaults to a ``gym.spaces.Box`` of the
                correct dimension with bounds at negative and positive
                infinity.
            reward: The reward can be specified in one of two
                ways: (1) by passing a callable with the signature
                ``value = reward(system, context)`` or (2) by passing a scalar
                vector-valued output port of ``simulator``'s system.
            action_port_id: The ID of an input port of ``simulator``'s system
                compatible with the ``action_space``.  Each Env *must* have an
                action port; passing ``None`` defaults to using the *first*
                input port (inspired by
                ``InputPortSelection.kUseFirstInputIfItExists``).
            observation_port_id: An output port of ``simulator``'s system
                compatible with the ``observation_space``. Each Env *must* have
                an observation port (it seems that gym doesn't support empty
                observation spaces / open-loop policies); passing ``None``
                defaults to using the *first* input port (inspired by
                ``OutputPortSelection.kUseFirstOutputIfItExists``).
            render_rgb_port_id: An optional output port of ``simulator``'s
                system that returns  an ``ImageRgba8U``; often the
                ``color_image`` port of a Drake ``RgbdSensor``.
            render_mode: The render mode of the environment determined at
                initialization. Defaults to ``human`` which uses visualizers
                inside the System (e.g. MeshcatVisualizer,
                PlanarSceneGraphVisualizer, etc.). ``render_mode`` equal to
                ``rgb_array`` evaluates the ``render_rgb_port`` and ``ansi``
                calls ``__repr__`` on the system Context.
            reset_handler: A function that sets the home state
                (plant, and/or env.) at ``reset()``.
                The reset state can be specified in one of
                the two ways:
                (if ``reset_handler`` is None) setting random context using
                a Drake random_generator
                (e.g. ``joint.set_random_pose_distribution()``
                using the ``reset()`` seed), (otherwise) using
                ``reset_handler()``.
            info_handler: A function that returns a ``dict[str, Any]``
                containing auxiliary diagnostic information (helpful for
                debugging, learning, and logging). Note: if ``step()``
                terminates with a ``RuntimeError``, then, to avoid
                unexpected behavior, `info_handler()`` will not be called
                and an empty info will be returned instead.
            hardware: If True, it prevents from setting random context at
                ``reset()`` when using ``random_generator``, but it does
                execute ``reset_handler()`` if given.

        Notes (using ``env`` as an instance of this class):

        - You may set simulator/integrator preferences by using
          ``env.simulator`` directly.
        - The ``done`` condition returned by ``step()`` is always False by
          default.  Use ``env.simulator.set_monitor()`` to use Drake's monitor
          functionality for specifying termination conditions.
        - You may additionally wish to directly set ``env.reward_range`` and/or
          ``env.spec``.  See the docs for ``gym.Env`` for more details.
        """
        super().__init__()
        if isinstance(simulator, Simulator):
            self.simulator = simulator
            self.make_simulator = None
        elif callable(simulator):
            self.simulator = None
            self.make_simulator = simulator
        else:
            raise ValueError("Invalid simulator argument")

        assert time_step > 0
        self.time_step = time_step

        assert isinstance(action_space, gym.spaces.Space)
        self.action_space = action_space

        assert isinstance(observation_space, gym.spaces.Space)
        self.observation_space = observation_space

        if isinstance(reward, (OutputPortIndex, str)):
            self.reward_port_id = reward
            self.reward = None
        elif callable(reward):
            self.reward_port_id = None
            self.reward = reward
        else:
            raise ValueError("Invalid reward argument")

        if action_port_id:
            assert isinstance(action_port_id, (InputPortIndex, str))
            self.action_port_id = action_port_id
        else:
            self.action_port_id = InputPortIndex(0)

        if observation_port_id:
            assert isinstance(observation_port_id, (OutputPortIndex, str))
            self.observation_port_id = observation_port_id
        else:
            self.observation_port_id = OutputPortIndex(0)

        self.metadata["render_modes"] = ["human", "ascii"]

        # Setup rendering.
        self.render_mode = render_mode
        if render_rgb_port_id:
            assert isinstance(render_rgb_port_id, (OutputPortIndex, str))
            self.render_mode = "rgb_array"
            self.metadata["render_modes"].append("rgb_array")
        self.render_rgb_port_id = render_rgb_port_id

        self.generator = RandomGenerator()

        if reset_handler is None or callable(reset_handler):
            self.reset_handler = reset_handler
        else:
            raise ValueError("reset_handler is not callable.")

        # Default return value of `info_handler()` is an empty `dict`.
        if info_handler is callable(info_handler):
            self.info_handler = info_handler
        else:
            self.info_handler = lambda _: dict()

        self.hardware = hardware

        if self.simulator:
            self._setup()

    def _setup(self):
        """Completes the setup once we have a self.simulator."""
        system = self.simulator.get_system()

        # Setup action port
        if self.action_port_id:
            if isinstance(self.action_port_id, InputPortIndex):
                self.action_port = system.get_input_port(self.action_port_id)
            else:
                self.action_port = system.GetInputPort(self.action_port_id)
        if self.action_port.get_data_type() == PortDataType.kVectorValued:
            assert np.array_equal(
                self.action_space.shape, [self.action_port.size()]
            )

        def get_output_port(id):
            if isinstance(id, OutputPortIndex):
                return system.get_output_port(id)
            return system.GetOutputPort(id)

        # Setup observation port
        if self.observation_port_id:
            self.observation_port = get_output_port(self.observation_port_id)
        if self.observation_port.get_data_type() == PortDataType.kVectorValued:
            assert np.array_equal(
                self.observation_space.shape, [self.observation_port.size()]
            )

        # Note: We require that there is no direct feedthrough action_port to
        # observation_port.  Unfortunately, HasDirectFeedthrough returns false
        # positives, and would produce noisy warnings.

        # Setup reward.
        if self.reward_port_id:
            reward_port = get_output_port(self.reward_port_id)
            self.reward = lambda system, context: reward_port.Eval(context)[0]

        # Setup rendering port.
        if self.render_rgb_port_id:
            self.render_rgb_port = get_output_port(self.render_rgb_port_id)
            assert (
                self.render_rgb_port.get_data_type()
                == PortDataType.kAbstractValued
            )
            assert isinstance(
                self.render_rgb_port.Allocate().get_value(), ImageRgba8U
            )

    def step(self, action):
        """
        Implements ``gym.Env.step`` to advance the simulation forward by one
        ``self.time_step``.

        Args:
            action: an element from ``self.action_space``.
        """
        assert self.simulator, "You must call reset() first"

        context = self.simulator.get_context()
        time = context.get_time()

        self.action_port.FixValue(context, action)
        truncated = False
        # Observation prior to advancing the simulation.
        prev_observation = self.observation_port.Eval(context)
        try:
            status = self.simulator.AdvanceTo(time + self.time_step)
        except RuntimeError as e:
            # TODO(JoseBarreiros-TRI) We don't currently check for the
            # error coming from the solver failing to converge.
            warnings.warn("Calling Done after catching RuntimeError:")
            warnings.warn(e.args[0])
            # Truncated is used when the solver failed to converge.
            # Note: this is different than the official use of truncated
            # in Gymnasium:
            # "Whether the truncation condition outside the scope of the MDP
            # is satisfied. Typically, this is a timelimit, but
            # could also be used to indicate an agent physically going out
            # of bounds."
            # We handle the solver failure to converge by returning
            # zero reward and the previous observation since the action
            # was not successfully applied. This comes at a cost of
            # an extra evaluation of the observation port.
            truncated = True
            terminated = False
            reward = 0
            # Do not call info handler, as the simulator has faulted.
            info = dict()

            return prev_observation, reward, terminated, truncated, info

        observation = self.observation_port.Eval(context)
        reward = self.reward(self.simulator.get_system(), context)
        terminated = not truncated and _reached_termination(status)
        info = self.info_handler(self.simulator)

        return observation, reward, terminated, truncated, info

    def reset(self, *, seed: int | None = None, options: dict | None = None):
        """
        If a callable "simulator factory" was passed to the constructor, then a
        new simulator is created.  Otherwise this method simply resets the
        ``simulator`` and its Context.
        """
        super().reset(seed=seed)
        assert options is None or options == dict(), (
            "Options are not supported in env.reset() method."
        )

        if seed is not None:
            # TODO(ggould) This should not reset the generator if it was
            # already explicitly seeded (see API spec), but we have no way to
            # check that at the moment.
            self.generator = RandomGenerator(seed)

        if self.make_simulator:
            self.simulator = self.make_simulator(self.generator)
            self._setup()

        context = self.simulator.get_mutable_context()
        context.SetTime(0)
        self.simulator.Initialize()
        if self.reset_handler is not None:
            # The initial state is set by reset_handler().
            self.simulator.get_system().SetDefaultContext(context)
            self.reset_handler(self.simulator, context, seed)
        else:
            if not self.hardware:
                self.simulator.get_system().SetRandomContext(
                    context, self.generator
                )

        # Note: The output port will be evaluated without fixing the input
        # port.
        observations = self.observation_port.Eval(context)
        info = self.info_handler(self.simulator)

        return observations, info

    def render(self):
        """
        Rendering in ``human`` mode is accomplished by calling ForcedPublish on
        ``system``.  This should cause visualizers inside the System (e.g.
        MeshcatVisualizer, PlanarSceneGraphVisualizer, etc.) to draw their
        outputs.  To be fully compliant, those visualizers should set their
        default publishing period to ``np.inf`` (do not publish periodically).

        Rendering in ``ascii`` mode calls ``__repr__`` on the system Context.

        Rendering in ``rgb_array`` mode is enabled by passing a compatible
        ``render_rgb_port`` to the class constructor.
        """
        assert self.simulator, "You must call reset() first"

        if self.render_mode == "human":
            self.simulator.get_system().ForcedPublish(
                self.simulator.get_context()
            )
            return
        elif self.render_mode == "ansi":
            return repr(self.simulator.get_context())
        elif self.render_mode == "rgb_array":
            assert self.render_rgb_port, (
                "You must set render_rgb_port in the constructor"
            )
            return self.render_rgb_port.Eval(self.simulator.get_context()).data[
                :, :, :3
            ]
        else:
            super(DrakeGymEnv).render()



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

class lcmt_call_python_data(object):
    """
    Generic data structure to wrap Python data types, for use with CallPython.
    For historical reasons, interface is modeled on mxArray (see
    https://www.mathworks.com/help/matlab/matlab_external/matlab-data.html).
    """

    __slots__ = ["data_type", "shape_type", "rows", "cols", "num_bytes", "data"]

    __typenames__ = ["int8_t", "int8_t", "int32_t", "int32_t", "int32_t", "byte"]

    __dimensions__ = [None, None, None, None, None, ["num_bytes"]]

    REMOTE_VARIABLE_REFERENCE = 0
    """ For data_type. """
    DOUBLE = 1
    CHAR = 2
    LOGICAL = 3
    INT = 4
    MATRIX = 0
    """ For shape_type. """
    VECTOR = 1
    SCALAR = 2

    def __init__(self):
        self.data_type = 0
        """ LCM Type: int8_t """
        self.shape_type = 0
        """ LCM Type: int8_t """
        self.rows = 0
        """ LCM Type: int32_t """
        self.cols = 0
        """ LCM Type: int32_t """
        self.num_bytes = 0
        """ LCM Type: int32_t """
        self.data = b""
        """ LCM Type: byte[num_bytes] """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_call_python_data._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">bbiii", self.data_type, self.shape_type, self.rows, self.cols, self.num_bytes))
        buf.write(bytearray(self.data[:self.num_bytes]))

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_call_python_data._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_call_python_data._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_call_python_data()
        self.data_type, self.shape_type, self.rows, self.cols, self.num_bytes = struct.unpack(">bbiii", buf.read(14))
        self.data = buf.read(self.num_bytes)
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_call_python_data in parents: return 0
        tmphash = (0xba5185e6611ca4e8) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_call_python_data._packed_fingerprint is None:
            lcmt_call_python_data._packed_fingerprint = struct.pack(">Q", lcmt_call_python_data._get_hash_recursive([]))
        return lcmt_call_python_data._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_call_python_data._get_packed_fingerprint())[0]




import collections.abc
import enum
import nanobind
import pydrake.symbolic
import scipy.sparse
from _typeshed import Incomplete
from pydrake.common.cpp_template import Binding as Binding
from typing import Any, Callable, ClassVar, overload

AddBilinearProductMcCormickEnvelopeSos2: nanobind.nb_func
AddLogarithmicSos1Constraint: nanobind.nb_func
AddLogarithmicSos2Constraint: nanobind.nb_func
AddSos2Constraint: nanobind.nb_func
ChooseBestSolver: nanobind.nb_func
GenerateSDPA: nanobind.nb_func
GetAvailableSolvers: nanobind.nb_func
GetProgramType: nanobind.nb_func
MakeFirstAvailableSolver: nanobind.nb_func
MakeSemidefiniteRelaxation: nanobind.nb_func
MakeSolver: nanobind.nb_func
Solve: nanobind.nb_func
SolveInParallel: nanobind.nb_func

class AugmentedLagrangianNonsmooth:
    def __init__(self, prog: MathematicalProgram, include_x_bounds: bool) -> None: ...
    def Eval(self, *args, **kwargs): ...
    def include_x_bounds(self) -> bool: ...
    def is_equality(self) -> list[bool]: ...
    def lagrangian_size(self) -> int: ...
    def prog(self) -> MathematicalProgram: ...
    def x_lo(self, *args, **kwargs): ...
    def x_up(self, *args, **kwargs): ...

class AugmentedLagrangianSmooth:
    def __init__(self, prog: MathematicalProgram, include_x_bounds: bool) -> None: ...
    def Eval(self, *args, **kwargs): ...
    def include_x_bounds(self) -> bool: ...
    def is_equality(self) -> list[bool]: ...
    def lagrangian_size(self) -> int: ...
    def prog(self) -> MathematicalProgram: ...
    def s_size(self) -> int: ...
    def x_lo(self, *args, **kwargs): ...
    def x_up(self, *args, **kwargs): ...

class Binding𝓣BoundingBoxConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: BoundingBoxConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> BoundingBoxConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTBoundingBoxConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣Constraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, c: Constraint, v) -> None: ...
    @overload
    def __init__(self, arg: object) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> Constraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣Cost𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, c: Cost, v) -> None: ...
    @overload
    def __init__(self, arg: object) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> Cost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTCostU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣EvaluatorBase𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, c: EvaluatorBase, v) -> None: ...
    @overload
    def __init__(self, arg: object) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> EvaluatorBase: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTEvaluatorBaseU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣ExponentialConeConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: ExponentialConeConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> ExponentialConeConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTExponentialConeConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣ExpressionConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: ExpressionConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> ExpressionConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTExpressionConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣ExpressionCost𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: ExpressionCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> ExpressionCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTExpressionCostU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣L1NormCost𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: L1NormCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> L1NormCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTL1NormCostU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣L2NormCost𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: L2NormCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> L2NormCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTL2NormCostU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣LInfNormCost𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LInfNormCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LInfNormCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLInfNormCostU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣LinearComplementarityConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearComplementarityConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearComplementarityConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearComplementarityConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣LinearConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣LinearCost𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearCostU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣LinearEqualityConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearEqualityConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearEqualityConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearEqualityConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣LinearMatrixInequalityConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearMatrixInequalityConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearMatrixInequalityConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearMatrixInequalityConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣LorentzConeConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LorentzConeConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LorentzConeConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLorentzConeConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣MinimumValueLowerBoundConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: MinimumValueLowerBoundConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> MinimumValueLowerBoundConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTMinimumValueLowerBoundConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣MinimumValueUpperBoundConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: MinimumValueUpperBoundConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> MinimumValueUpperBoundConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTMinimumValueUpperBoundConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣PerspectiveQuadraticCost𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: PerspectiveQuadraticCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> PerspectiveQuadraticCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTPerspectiveQuadraticCostU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣PositiveSemidefiniteConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: PositiveSemidefiniteConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> PositiveSemidefiniteConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTPositiveSemidefiniteConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣QuadraticConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: QuadraticConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> QuadraticConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTQuadraticConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣QuadraticCost𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: QuadraticCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> QuadraticCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTQuadraticCostU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣RotatedLorentzConeConstraint𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: RotatedLorentzConeConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> RotatedLorentzConeConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTRotatedLorentzConeConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class Binding𝓣VisualizationCallback𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: VisualizationCallback, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> VisualizationCallback: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTVisualizationCallbackU) -> bool: ...
    def __hash__(self) -> int: ...

class BoundingBoxConstraint(LinearConstraint):
    def __init__(self, lb, ub) -> None: ...

class ClarabelSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class ClarabelSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def iterations(self) -> int: ...
    @property
    def solve_time(self) -> float: ...
    @property
    def status(self) -> str: ...

class ClpSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class ClpSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def status(self) -> int: ...

class CommonSolverOption(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kMaxThreads: ClassVar[CommonSolverOption] = ...
    kPrintFileName: ClassVar[CommonSolverOption] = ...
    kPrintToConsole: ClassVar[CommonSolverOption] = ...
    kStandaloneReproductionFileName: ClassVar[CommonSolverOption] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class Constraint(EvaluatorBase):
    def __init__(self, *args, **kwargs) -> None: ...
    @overload
    def CheckSatisfied(self, x, tol: float = ...) -> bool: ...
    @overload
    def CheckSatisfied(self, x, tol: float = ...) -> bool: ...
    @overload
    def CheckSatisfied(self, x) -> pydrake.symbolic.Formula: ...
    def CheckSatisfiedVectorized(self, x, tol: float) -> list[bool]: ...
    def lower_bound(self, *args, **kwargs): ...
    def num_constraints(self) -> int: ...
    def upper_bound(self, *args, **kwargs): ...

class Cost(EvaluatorBase):
    def __init__(self, *args, **kwargs) -> None: ...

class CsdpSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class CsdpSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def Z_val(self): ...
    @property
    def dual_objective(self) -> float: ...
    @property
    def primal_objective(self) -> float: ...
    @property
    def return_code(self) -> int: ...
    @property
    def y_val(self): ...

class EvaluatorBase:
    def __init__(self, *args, **kwargs) -> None: ...
    def Eval(self, *args, **kwargs): ...
    def SetGradientSparsityPattern(self, gradient_sparsity_pattern: collections.abc.Sequence[tuple[int, int]]) -> None: ...
    @overload
    def ToLatex(self, vars, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def get_description(self) -> str: ...
    def gradient_sparsity_pattern(self) -> list[tuple[int, int]] | None: ...
    def is_thread_safe(self) -> bool: ...
    def num_outputs(self) -> int: ...
    def num_vars(self) -> int: ...
    def set_description(self, arg: str) -> None: ...

class ExponentialConeConstraint(Constraint):
    def __init__(self, A: scipy.sparse.csc_matrix[float], b) -> None: ...
    def A(self, *args, **kwargs): ...
    def b(self, *args, **kwargs): ...

class ExpressionConstraint(Constraint):
    def __init__(self, v, lb, ub) -> None: ...
    def expressions(self, *args, **kwargs): ...
    def vars(self, *args, **kwargs): ...

class ExpressionCost(Cost):
    def __init__(self, e: pydrake.symbolic.Expression) -> None: ...
    def expression(self) -> pydrake.symbolic.Expression: ...
    def vars(self, *args, **kwargs): ...

class GurobiSolver(SolverInterface):
    class License:
        def __init__(self, *args, **kwargs) -> None: ...
        def is_valid(self) -> bool: ...
        def __enter__(self) -> GurobiSolver.License: ...
        def __exit__(self, *args, **kwargs) -> None: ...

    class SolveStatusInfo:
        def __init__(self, *args, **kwargs) -> None: ...
        @property
        def best_bound(self) -> float: ...
        @property
        def best_objective(self) -> float: ...
        @property
        def current_objective(self) -> float: ...
        @property
        def explored_node_count(self) -> int: ...
        @property
        def feasible_solutions_count(self) -> int: ...
        @property
        def reported_runtime(self) -> float: ...
    AcquireLicense: ClassVar[nanobind.nb_func] = ...
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...
    def AddMipNodeCallback(self, callback) -> None: ...
    def AddMipSolCallback(self, callback: collections.abc.Callable[[MathematicalProgram, GurobiSolver.SolveStatusInfo], None]) -> None: ...

class GurobiSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def error_code(self) -> int: ...
    @property
    def objective_bound(self) -> float: ...
    @property
    def optimization_status(self) -> int: ...
    @property
    def optimizer_time(self) -> float: ...

class IntervalBinning(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kLinear: ClassVar[IntervalBinning] = ...
    kLogarithmic: ClassVar[IntervalBinning] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class IpoptSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class IpoptSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    def ConvertStatusToString(self) -> str: ...
    @property
    def g(self): ...
    @property
    def lambda_val(self): ...
    @property
    def status(self) -> int: ...
    @property
    def z_L(self): ...
    @property
    def z_U(self): ...

class L1NormCost(Cost):
    def __init__(self, A, b) -> None: ...
    def A(self, *args, **kwargs): ...
    def UpdateCoefficients(self, new_A, new_b=...) -> None: ...
    def b(self, *args, **kwargs): ...
    def update_A_entry(self, i: int, j: int, val: float) -> None: ...
    def update_b_entry(self, i: int, val: float) -> None: ...

class L2NormCost(Cost):
    @overload
    def __init__(self, A, b) -> None: ...
    @overload
    def __init__(self, A: scipy.sparse.csc_matrix[float], b) -> None: ...
    def GetDenseA(self, *args, **kwargs): ...
    @overload
    def UpdateCoefficients(self, new_A, new_b=...) -> None: ...
    @overload
    def UpdateCoefficients(self, new_A: scipy.sparse.csc_matrix[float], new_b=...) -> None: ...
    def b(self, *args, **kwargs): ...
    def get_sparse_A(self) -> scipy.sparse.csc_matrix[float]: ...

class LInfNormCost(Cost):
    def __init__(self, A, b) -> None: ...
    def A(self, *args, **kwargs): ...
    def UpdateCoefficients(self, new_A, new_b=...) -> None: ...
    def b(self, *args, **kwargs): ...
    def update_A_entry(self, i: int, j: int, val: float) -> None: ...
    def update_b_entry(self, i: int, val: float) -> None: ...

class LinearComplementarityConstraint(Constraint):
    def __init__(self, *args, **kwargs) -> None: ...
    def M(self, *args, **kwargs): ...
    def q(self, *args, **kwargs): ...

class LinearConstraint(Constraint):
    @overload
    def __init__(self, A, lb, ub) -> None: ...
    @overload
    def __init__(self, A: scipy.sparse.csc_matrix[float], lb, ub) -> None: ...
    def GetDenseA(self, *args, **kwargs): ...
    def RemoveTinyCoefficient(self, tol: float) -> None: ...
    @overload
    def UpdateCoefficients(self, new_A, new_lb, new_ub) -> None: ...
    @overload
    def UpdateCoefficients(self, new_A: scipy.sparse.csc_matrix[float], new_lb, new_ub) -> None: ...
    def UpdateLowerBound(self, new_lb) -> None: ...
    def UpdateUpperBound(self, new_ub) -> None: ...
    def get_sparse_A(self) -> scipy.sparse.csc_matrix[float]: ...
    def is_dense_A_constructed(self) -> bool: ...
    def set_bounds(self, new_lb, new_ub) -> None: ...

class LinearCost(Cost):
    def __init__(self, a, b: float) -> None: ...
    def UpdateCoefficients(self, new_a, new_b: float = ...) -> None: ...
    def a(self, *args, **kwargs): ...
    def b(self) -> float: ...
    def update_coefficient_entry(self, i: int, val: float) -> None: ...
    def update_constant_term(self, new_b: float) -> None: ...

class LinearEqualityConstraint(LinearConstraint):
    @overload
    def __init__(self, Aeq, beq) -> None: ...
    @overload
    def __init__(self, Aeq: scipy.sparse.csc_matrix[float], beq) -> None: ...
    @overload
    def __init__(self, a, beq: float) -> None: ...
    @overload
    def UpdateCoefficients(self, Aeq, beq) -> None: ...
    @overload
    def UpdateCoefficients(self, Aeq: scipy.sparse.csc_matrix[float], beq) -> None: ...

class LinearMatrixInequalityConstraint(Constraint):
    def __init__(self, F, symmetry_tolerance: float = ...) -> None: ...
    def F(self, *args, **kwargs): ...
    @overload
    def matrix_rows(self) -> int: ...
    @overload
    def matrix_rows(self) -> Any: ...

class LorentzConeConstraint(Constraint):
    class EvalType(enum.Enum):
        __new__: ClassVar[Callable] = ...
        _generate_next_value_: ClassVar[Callable] = ...
        _hashable_values_: ClassVar[list] = ...
        _member_map_: ClassVar[dict] = ...
        _member_names_: ClassVar[list] = ...
        _member_type_: ClassVar[type[object]] = ...
        _unhashable_values_: ClassVar[list] = ...
        _unhashable_values_map_: ClassVar[dict] = ...
        _use_args_: ClassVar[bool] = ...
        _value2member_map_: ClassVar[dict] = ...
        _value_repr_: ClassVar[None] = ...
        kConvex: ClassVar[LorentzConeConstraint.EvalType] = ...
        kConvexSmooth: ClassVar[LorentzConeConstraint.EvalType] = ...
        kNonconvex: ClassVar[LorentzConeConstraint.EvalType] = ...
        __nb_enum__: ClassVar[PyCapsule] = ...
    def __init__(self, A, b, eval_type: LorentzConeConstraint.EvalType = ...) -> None: ...
    def A(self) -> scipy.sparse.csc_matrix[float]: ...
    def UpdateCoefficients(self, new_A, new_b) -> None: ...
    def b(self, *args, **kwargs): ...
    def eval_type(self) -> LorentzConeConstraint.EvalType: ...

class MathematicalProgram:
    class NonnegativePolynomial(enum.Enum):
        __new__: ClassVar[Callable] = ...
        _generate_next_value_: ClassVar[Callable] = ...
        _hashable_values_: ClassVar[list] = ...
        _member_map_: ClassVar[dict] = ...
        _member_names_: ClassVar[list] = ...
        _member_type_: ClassVar[type[object]] = ...
        _unhashable_values_: ClassVar[list] = ...
        _unhashable_values_map_: ClassVar[dict] = ...
        _use_args_: ClassVar[bool] = ...
        _value2member_map_: ClassVar[dict] = ...
        _value_repr_: ClassVar[None] = ...
        kDsos: ClassVar[MathematicalProgram.NonnegativePolynomial] = ...
        kSdsos: ClassVar[MathematicalProgram.NonnegativePolynomial] = ...
        kSos: ClassVar[MathematicalProgram.NonnegativePolynomial] = ...
        __nb_enum__: ClassVar[PyCapsule] = ...
    def __init__(self) -> None: ...
    def Add2NormSquaredCost(self, A, b, vars) -> BindingTQuadraticCostU: ...
    def AddBoundingBoxConstraint(self, arg0, arg1, arg2) -> BindingTBoundingBoxConstraintU: ...
    @overload
    def AddConstraint(self, func: collections.abc.Callable, lb, ub, vars, description: str = ...) -> BindingTConstraintU: ...
    @overload
    def AddConstraint(self, arg0: pydrake.symbolic.Expression, arg1: float, arg2: float) -> BindingTConstraintU: ...
    @overload
    def AddConstraint(self, constraint: Constraint, vars) -> BindingTConstraintU: ...
    @overload
    def AddConstraint(self, formulas) -> BindingTConstraintU: ...
    @overload
    def AddConstraint(self, binding: BindingTConstraintU) -> BindingTConstraintU: ...
    @overload
    def AddCost(self, func: collections.abc.Callable, vars, description: str = ...) -> BindingTCostU: ...
    @overload
    def AddCost(self, binding: BindingTCostU) -> BindingTCostU: ...
    @overload
    def AddCost(self, e: pydrake.symbolic.Expression) -> BindingTCostU: ...
    @overload
    def AddCost(self, obj: Cost, vars) -> BindingTCostU: ...
    def AddDecisionVariables(self, decision_variables) -> None: ...
    def AddEqualityConstraintBetweenPolynomials(self, p1: pydrake.symbolic.Polynomial, p2: pydrake.symbolic.Polynomial) -> list[BindingTLinearEqualityConstraintU]: ...
    @overload
    def AddExponentialConeConstraint(self, A, b, vars) -> BindingTExponentialConeConstraintU: ...
    @overload
    def AddExponentialConeConstraint(self, z) -> BindingTExponentialConeConstraintU: ...
    def AddIndeterminate(self, new_indeterminate: pydrake.symbolic.Variable) -> int: ...
    @overload
    def AddIndeterminates(self, new_indeterminates) -> None: ...
    @overload
    def AddIndeterminates(self, new_indeterminates: pydrake.symbolic.Variables) -> None: ...
    def AddL1NormCostInEpigraphForm(self, *args, **kwargs): ...
    @overload
    def AddL2NormCost(self, A, b, vars) -> BindingTL2NormCostU: ...
    @overload
    def AddL2NormCost(self, e: pydrake.symbolic.Expression, psd_tol: float = ..., coefficient_tol: float = ...) -> BindingTL2NormCostU: ...
    def AddL2NormCostUsingConicConstraint(self, A, b, vars) -> tuple[pydrake.symbolic.Variable, BindingTLinearCostU, BindingTLorentzConeConstraintU]: ...
    def AddLinearComplementarityConstraint(self, M, q, vars) -> BindingTLinearComplementarityConstraintU: ...
    @overload
    def AddLinearConstraint(self, A, lb, ub, vars) -> BindingTLinearConstraintU: ...
    @overload
    def AddLinearConstraint(self, a, lb: float, ub: float, vars) -> BindingTLinearConstraintU: ...
    @overload
    def AddLinearConstraint(self, A: scipy.sparse.csc_matrix[float], lb, ub, vars) -> BindingTLinearConstraintU: ...
    @overload
    def AddLinearConstraint(self, e: pydrake.symbolic.Expression, lb: float, ub: float) -> BindingTLinearConstraintU: ...
    @overload
    def AddLinearConstraint(self, v, lb, ub) -> BindingTLinearConstraintU: ...
    @overload
    def AddLinearConstraint(self, f: pydrake.symbolic.Formula) -> BindingTLinearConstraintU: ...
    @overload
    def AddLinearConstraint(self, formulas) -> BindingTLinearConstraintU: ...
    @overload
    def AddLinearCost(self, e: pydrake.symbolic.Expression) -> BindingTLinearCostU: ...
    @overload
    def AddLinearCost(self, a, b: float, vars) -> BindingTLinearCostU: ...
    @overload
    def AddLinearCost(self, a, vars) -> BindingTLinearCostU: ...
    @overload
    def AddLinearEqualityConstraint(self, Aeq, beq, vars) -> BindingTLinearEqualityConstraintU: ...
    @overload
    def AddLinearEqualityConstraint(self, a, beq: float, vars) -> BindingTLinearEqualityConstraintU: ...
    @overload
    def AddLinearEqualityConstraint(self, Aeq: scipy.sparse.csc_matrix[float], beq, vars) -> BindingTLinearEqualityConstraintU: ...
    @overload
    def AddLinearEqualityConstraint(self, e: pydrake.symbolic.Expression, b: float) -> BindingTLinearEqualityConstraintU: ...
    @overload
    def AddLinearEqualityConstraint(self, f: pydrake.symbolic.Formula) -> BindingTLinearEqualityConstraintU: ...
    @overload
    def AddLinearEqualityConstraint(self, formulas) -> BindingTLinearEqualityConstraintU: ...
    @overload
    def AddLinearEqualityConstraint(self, v, b) -> BindingTLinearEqualityConstraintU: ...
    @overload
    def AddLinearMatrixInequalityConstraint(self, F, vars) -> BindingTLinearMatrixInequalityConstraintU: ...
    @overload
    def AddLinearMatrixInequalityConstraint(self, X) -> BindingTLinearMatrixInequalityConstraintU: ...
    def AddLogDeterminantLowerBoundConstraint(self, *args, **kwargs): ...
    @overload
    def AddLorentzConeConstraint(self, f: pydrake.symbolic.Formula, eval_type: LorentzConeConstraint.EvalType = ..., psd_tol: float = ..., coefficient_tol: float = ...) -> BindingTLorentzConeConstraintU: ...
    @overload
    def AddLorentzConeConstraint(self, v, eval_type: LorentzConeConstraint.EvalType = ...) -> BindingTLorentzConeConstraintU: ...
    @overload
    def AddLorentzConeConstraint(self, linear_expression: pydrake.symbolic.Expression, quadratic_expression: pydrake.symbolic.Expression, tol: float = ..., eval_type: LorentzConeConstraint.EvalType = ...) -> BindingTLorentzConeConstraintU: ...
    @overload
    def AddLorentzConeConstraint(self, A, b, vars, eval_type: LorentzConeConstraint.EvalType = ...) -> BindingTLorentzConeConstraintU: ...
    @overload
    def AddMaximizeGeometricMeanCost(self, A, b, x) -> BindingTLinearCostU: ...
    @overload
    def AddMaximizeGeometricMeanCost(self, x, c: float) -> BindingTLinearCostU: ...
    def AddMaximizeLogDeterminantCost(self, *args, **kwargs): ...
    @overload
    def AddPositiveDiagonallyDominantDualConeMatrixConstraint(self, X) -> BindingTLinearConstraintU: ...
    @overload
    def AddPositiveDiagonallyDominantDualConeMatrixConstraint(self, X) -> BindingTLinearConstraintU: ...
    def AddPositiveDiagonallyDominantMatrixConstraint(self, *args, **kwargs): ...
    @overload
    def AddPositiveSemidefiniteConstraint(self, arg) -> BindingTPositiveSemidefiniteConstraintU: ...
    @overload
    def AddPositiveSemidefiniteConstraint(self, e) -> Any: ...
    def AddPrincipalSubmatrixIsPsdConstraint(self, arg0, arg1: collections.abc.Set[int]) -> BindingTPositiveSemidefiniteConstraintU: ...
    def AddQuadraticAsRotatedLorentzConeConstraint(self, Q, b, c: float, vars, psd_tol: float = ...) -> BindingTRotatedLorentzConeConstraintU: ...
    @overload
    def AddQuadraticConstraint(self, Q, b, lb: float, ub: float, vars, hessian_type: QuadraticConstraint.HessianType | None = ...) -> BindingTQuadraticConstraintU: ...
    @overload
    def AddQuadraticConstraint(self, e: pydrake.symbolic.Expression, lb: float, ub: float, hessian_type: QuadraticConstraint.HessianType | None = ...) -> BindingTQuadraticConstraintU: ...
    @overload
    def AddQuadraticCost(self, e: pydrake.symbolic.Expression, is_convex: bool | None = ...) -> BindingTQuadraticCostU: ...
    @overload
    def AddQuadraticCost(self, Q, b, vars, is_convex: bool | None = ...) -> BindingTQuadraticCostU: ...
    @overload
    def AddQuadraticCost(self, Q, b, c: float, vars, is_convex: bool | None = ...) -> BindingTQuadraticCostU: ...
    @overload
    def AddQuadraticErrorCost(self, w: float, x_desired, vars) -> BindingTQuadraticCostU: ...
    @overload
    def AddQuadraticErrorCost(self, Q, x_desired, vars) -> BindingTQuadraticCostU: ...
    @overload
    def AddRotatedLorentzConeConstraint(self, linear_expression1: pydrake.symbolic.Expression, linear_expression2: pydrake.symbolic.Expression, quadratic_expression: pydrake.symbolic.Expression, tol: float = ...) -> BindingTRotatedLorentzConeConstraintU: ...
    @overload
    def AddRotatedLorentzConeConstraint(self, v) -> BindingTRotatedLorentzConeConstraintU: ...
    @overload
    def AddRotatedLorentzConeConstraint(self, A, b, vars) -> BindingTRotatedLorentzConeConstraintU: ...
    @overload
    def AddScaledDiagonallyDominantDualConeMatrixConstraint(self, X) -> list[BindingTRotatedLorentzConeConstraintU]: ...
    @overload
    def AddScaledDiagonallyDominantDualConeMatrixConstraint(self, X) -> list[BindingTRotatedLorentzConeConstraintU]: ...
    def AddScaledDiagonallyDominantMatrixConstraint(self, *args, **kwargs): ...
    def AddSosConstraint(self, *args, **kwargs): ...
    def AddVisualizationCallback(self, arg0, arg1) -> BindingTVisualizationCallbackU: ...
    @overload
    def CheckSatisfied(self, binding: BindingTConstraintU, prog_var_vals, tol: float = ...) -> bool: ...
    @overload
    def CheckSatisfied(self, bindings: collections.abc.Sequence[BindingTConstraintU], prog_var_vals, tol: float = ...) -> bool: ...
    @overload
    def CheckSatisfiedAtInitialGuess(self, binding: BindingTConstraintU, tol: float = ...) -> bool: ...
    @overload
    def CheckSatisfiedAtInitialGuess(self, bindings: collections.abc.Sequence[BindingTConstraintU], tol: float = ...) -> bool: ...
    def ClearVariableScaling(self) -> None: ...
    def Clone(self) -> MathematicalProgram: ...
    def EvalBinding(self, *args, **kwargs): ...
    def EvalBindingAtInitialGuess(self, *args, **kwargs): ...
    def EvalBindingVectorized(self, *args, **kwargs): ...
    def EvalBindings(self, *args, **kwargs): ...
    def FindDecisionVariableIndex(self, var: pydrake.symbolic.Variable) -> int: ...
    def FindDecisionVariableIndices(self, vars) -> list[int]: ...
    def FindIndeterminateIndex(self, var: pydrake.symbolic.Variable) -> int: ...
    def GetAllConstraints(self) -> list[BindingTConstraintU]: ...
    def GetAllCosts(self) -> list[BindingTCostU]: ...
    def GetBindingVariableValues(self, *args, **kwargs): ...
    def GetInitialGuess(self, arg: pydrake.symbolic.Variable) -> float: ...
    def GetLinearConstraints(self) -> list[BindingTLinearConstraintU]: ...
    def GetVariableScaling(self) -> dict[int, float]: ...
    def IsThreadSafe(self) -> bool: ...
    def MakePolynomial(self, e: pydrake.symbolic.Expression) -> pydrake.symbolic.Polynomial: ...
    def NewBinaryVariables(self, *args, **kwargs): ...
    def NewContinuousVariables(self, *args, **kwargs): ...
    def NewEvenDegreeDsosPolynomial(self, *args, **kwargs): ...
    def NewEvenDegreeFreePolynomial(self, indeterminates: pydrake.symbolic.Variables, degree: int, coeff_name: str = ...) -> pydrake.symbolic.Polynomial: ...
    def NewEvenDegreeNonnegativePolynomial(self, *args, **kwargs): ...
    def NewEvenDegreeSdsosPolynomial(self, *args, **kwargs): ...
    def NewEvenDegreeSosPolynomial(self, *args, **kwargs): ...
    def NewFreePolynomial(self, indeterminates: pydrake.symbolic.Variables, deg: int, coeff_name: str = ...) -> pydrake.symbolic.Polynomial: ...
    def NewIndeterminates(self, *args, **kwargs): ...
    def NewOddDegreeFreePolynomial(self, indeterminates: pydrake.symbolic.Variables, degree: int, coeff_name: str = ...) -> pydrake.symbolic.Polynomial: ...
    def NewSosPolynomial(self, gramian, monomial_basis, type: MathematicalProgram.NonnegativePolynomial = ...) -> pydrake.symbolic.Polynomial: ...
    def NewSymmetricContinuousVariables(self, *args, **kwargs): ...
    def RelaxPsdConstraintToDdDualCone(self, constraint: BindingTPositiveSemidefiniteConstraintU) -> BindingTLinearConstraintU: ...
    def RelaxPsdConstraintToSddDualCone(self, constraint: BindingTPositiveSemidefiniteConstraintU) -> list[BindingTRotatedLorentzConeConstraintU]: ...
    def RemoveConstraint(self, constraint: BindingTConstraintU) -> int: ...
    def RemoveCost(self, cost: BindingTCostU) -> int: ...
    def RemoveDecisionVariable(self, var: pydrake.symbolic.Variable) -> int: ...
    def RemoveVisualizationCallback(self, callback: BindingTVisualizationCallbackU) -> int: ...
    def Reparse(self, p: pydrake.symbolic.Polynomial) -> None: ...
    @overload
    def SetDecisionVariableValueInVector(self, decision_variable: pydrake.symbolic.Variable, decision_variable_new_value: float, values) -> None: ...
    @overload
    def SetDecisionVariableValueInVector(self, decision_variables, decision_variables_new_values, values) -> None: ...
    def SetInitialGuess(self, arg0: pydrake.symbolic.Variable, arg1: float) -> None: ...
    def SetInitialGuessForAllVariables(self, arg) -> None: ...
    @overload
    def SetSolverOption(self, solver_id: SolverId, solver_option: str, option_value: float) -> None: ...
    @overload
    def SetSolverOption(self, solver_id: SolverId, solver_option: str, option_value: int) -> None: ...
    @overload
    def SetSolverOption(self, solver_id: SolverId, solver_option: str, option_value: str) -> None: ...
    @overload
    def SetSolverOption(self, arg0: SolverType, arg1: str, arg2: float) -> None: ...
    def SetSolverOptions(self, arg: SolverOptions) -> None: ...
    def SetVariableScaling(self, var: pydrake.symbolic.Variable, s: float) -> None: ...
    def TightenPsdConstraintToDd(self, *args, **kwargs): ...
    def TightenPsdConstraintToSdd(self, *args, **kwargs): ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def bounding_box_constraints(self) -> list[BindingTBoundingBoxConstraintU]: ...
    def decision_variable(self, i: int) -> pydrake.symbolic.Variable: ...
    def decision_variable_index(self) -> dict[int, int]: ...
    def decision_variables(self, *args, **kwargs): ...
    def exponential_cone_constraints(self) -> list[BindingTExponentialConeConstraintU]: ...
    def generic_constraints(self) -> list[BindingTConstraintU]: ...
    def generic_costs(self) -> list[BindingTCostU]: ...
    def indeterminate(self, i: int) -> pydrake.symbolic.Variable: ...
    def indeterminates(self, *args, **kwargs): ...
    def indeterminates_index(self) -> dict[int, int]: ...
    def initial_guess(self, *args, **kwargs): ...
    def l2norm_costs(self) -> list[BindingTL2NormCostU]: ...
    def linear_complementarity_constraints(self) -> list[BindingTLinearComplementarityConstraintU]: ...
    def linear_constraints(self) -> list[BindingTLinearConstraintU]: ...
    def linear_costs(self) -> list[BindingTLinearCostU]: ...
    def linear_equality_constraints(self) -> list[BindingTLinearEqualityConstraintU]: ...
    def linear_matrix_inequality_constraints(self) -> list[BindingTLinearMatrixInequalityConstraintU]: ...
    def lorentz_cone_constraints(self) -> list[BindingTLorentzConeConstraintU]: ...
    def num_indeterminates(self) -> int: ...
    def num_vars(self) -> int: ...
    def positive_semidefinite_constraints(self) -> list[BindingTPositiveSemidefiniteConstraintU]: ...
    def quadratic_constraints(self) -> list[BindingTQuadraticConstraintU]: ...
    def quadratic_costs(self) -> list[BindingTQuadraticCostU]: ...
    def required_capabilities(self) -> set[ProgramAttribute]: ...
    def rotated_lorentz_cone_constraints(self) -> list[BindingTRotatedLorentzConeConstraintU]: ...
    def solver_options(self) -> SolverOptions: ...
    def visualization_callbacks(self) -> list[BindingTVisualizationCallbackU]: ...
    def __copy__(self) -> MathematicalProgram: ...
    def __deepcopy__(self, arg: dict) -> MathematicalProgram: ...

class MathematicalProgramResult:
    def __init__(self) -> None: ...
    def EvalBinding(self, *args, **kwargs): ...
    def GetDualSolution(self, constraint) -> Any: ...
    def GetInfeasibleConstraintNames(self, prog: MathematicalProgram, tol: float | None = ...) -> list[str]: ...
    def GetInfeasibleConstraints(self, prog: MathematicalProgram, tol: float | None = ...) -> list[BindingTConstraintU]: ...
    def GetSolution(self, arg: pydrake.symbolic.Variable) -> float: ...
    def GetSuboptimalSolution(self, arg0: pydrake.symbolic.Variable, arg1: int) -> float: ...
    def SetSolution(self, var: pydrake.symbolic.Variable, value: float) -> None: ...
    def get_optimal_cost(self) -> float: ...
    def get_solution_result(self) -> SolutionResult: ...
    def get_solver_details(self) -> object: ...
    def get_solver_id(self) -> SolverId: ...
    def get_suboptimal_objective(self, solution_number: int) -> float: ...
    def get_x_val(self, *args, **kwargs): ...
    def is_success(self) -> bool: ...
    def num_suboptimal_solution(self) -> int: ...
    def set_solution_result(self, arg: SolutionResult) -> None: ...
    def set_x_val(self, x_val) -> None: ...

class MinimumValueLowerBoundConstraint(Constraint):
    def __init__(self, num_vars: int, minimum_value_lower: float, influence_value_offset: float, max_num_values: int, value_function, value_function_double=...) -> None: ...
    def influence_value(self) -> float: ...
    def minimum_value_lower(self) -> float: ...
    def set_penalty_function(self, new_penalty_function: collections.abc.Callable[[float, bool], tuple]) -> None: ...

class MinimumValueUpperBoundConstraint(Constraint):
    def __init__(self, num_vars: int, minimum_value_upper: float, influence_value_offset: float, max_num_values: int, value_function, value_function_double=...) -> None: ...
    def influence_value(self) -> float: ...
    def minimum_value_upper(self) -> float: ...
    def set_penalty_function(self, new_penalty_function: collections.abc.Callable[[float, bool], tuple]) -> None: ...

class MixedIntegerBranchAndBound:
    class Options:
        max_explored_nodes: int
        def __init__(self, **kwargs) -> None: ...
        def __copy__(self) -> MixedIntegerBranchAndBound.Options: ...
        def __deepcopy__(self, arg: dict) -> MixedIntegerBranchAndBound.Options: ...
        @property
        def __fields__(self): ...
    def __init__(self, prog: MathematicalProgram, solver_id: SolverId, options: MixedIntegerBranchAndBound.Options = ...) -> None: ...
    def GetOptimalCost(self) -> float: ...
    def GetSolution(self, mip_var: pydrake.symbolic.Variable, nth_best_solution: int = ...) -> float: ...
    def GetSubOptimalCost(self, nth_suboptimal_cost: int) -> float: ...
    def Solve(self) -> SolutionResult: ...

class MixedIntegerRotationConstraintGenerator:
    class Approach(enum.Enum):
        __new__: ClassVar[Callable] = ...
        _generate_next_value_: ClassVar[Callable] = ...
        _hashable_values_: ClassVar[list] = ...
        _member_map_: ClassVar[dict] = ...
        _member_names_: ClassVar[list] = ...
        _member_type_: ClassVar[type[object]] = ...
        _unhashable_values_: ClassVar[list] = ...
        _unhashable_values_map_: ClassVar[dict] = ...
        _use_args_: ClassVar[bool] = ...
        _value2member_map_: ClassVar[dict] = ...
        _value_repr_: ClassVar[None] = ...
        kBilinearMcCormick: ClassVar[MixedIntegerRotationConstraintGenerator.Approach] = ...
        kBoth: ClassVar[MixedIntegerRotationConstraintGenerator.Approach] = ...
        kBoxSphereIntersection: ClassVar[MixedIntegerRotationConstraintGenerator.Approach] = ...
        __nb_enum__: ClassVar[PyCapsule] = ...

    class ReturnType:
        def __init__(self, *args, **kwargs) -> None: ...
        @property
        def B_(self): ...
        @property
        def lambda_(self): ...
    def __init__(self, approach: MixedIntegerRotationConstraintGenerator.Approach, num_intervals_per_half_axis: int, interval_binning: IntervalBinning) -> None: ...
    def AddToProgram(self, R, prog: MathematicalProgram) -> MixedIntegerRotationConstraintGenerator.ReturnType: ...
    def interval_binning(self) -> IntervalBinning: ...
    def num_intervals_per_half_axis(self) -> int: ...
    def phi(self, *args, **kwargs): ...
    def phi_nonnegative(self, *args, **kwargs): ...

class MobyLcpSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class MosekSolver(SolverInterface):
    class License:
        def __init__(self, *args, **kwargs) -> None: ...
        def is_valid(self) -> bool: ...
        def __enter__(self) -> MosekSolver.License: ...
        def __exit__(self, *args, **kwargs) -> None: ...
    AcquireLicense: ClassVar[nanobind.nb_func] = ...
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class MosekSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def optimizer_time(self) -> float: ...
    @property
    def rescode(self) -> int: ...
    @property
    def solution_status(self) -> int: ...

class NloptSolver(SolverInterface):
    AlgorithmName: ClassVar[nanobind.nb_func] = ...
    ConstraintToleranceName: ClassVar[nanobind.nb_func] = ...
    MaxEvalName: ClassVar[nanobind.nb_func] = ...
    MaxTimeName: ClassVar[nanobind.nb_func] = ...
    XAbsoluteToleranceName: ClassVar[nanobind.nb_func] = ...
    XRelativeToleranceName: ClassVar[nanobind.nb_func] = ...
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class NloptSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def status(self) -> int: ...

class OsqpSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class OsqpSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def dual_res(self) -> float: ...
    @property
    def iter(self) -> int: ...
    @property
    def polish_time(self) -> float: ...
    @property
    def primal_res(self) -> float: ...
    @property
    def run_time(self) -> float: ...
    @property
    def setup_time(self) -> float: ...
    @property
    def solve_time(self) -> float: ...
    @property
    def status_val(self) -> int: ...
    @property
    def y(self): ...

class PerspectiveQuadraticCost(Cost):
    def __init__(self, A, b) -> None: ...
    def A(self, *args, **kwargs): ...
    def UpdateCoefficients(self, new_A, new_b) -> None: ...
    def b(self, *args, **kwargs): ...
    def update_A_entry(self, i: int, j: int, val: float) -> None: ...
    def update_b_entry(self, i: int, val: float) -> None: ...

class PositiveSemidefiniteConstraint(Constraint):
    def __init__(self, rows: int) -> None: ...
    def matrix_rows(self) -> int: ...

class ProgramAttribute(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kBinaryVariable: ClassVar[ProgramAttribute] = ...
    kCallback: ClassVar[ProgramAttribute] = ...
    kExponentialConeConstraint: ClassVar[ProgramAttribute] = ...
    kGenericConstraint: ClassVar[ProgramAttribute] = ...
    kGenericCost: ClassVar[ProgramAttribute] = ...
    kL2NormCost: ClassVar[ProgramAttribute] = ...
    kLinearComplementarityConstraint: ClassVar[ProgramAttribute] = ...
    kLinearConstraint: ClassVar[ProgramAttribute] = ...
    kLinearCost: ClassVar[ProgramAttribute] = ...
    kLinearEqualityConstraint: ClassVar[ProgramAttribute] = ...
    kLorentzConeConstraint: ClassVar[ProgramAttribute] = ...
    kPositiveSemidefiniteConstraint: ClassVar[ProgramAttribute] = ...
    kQuadraticConstraint: ClassVar[ProgramAttribute] = ...
    kQuadraticCost: ClassVar[ProgramAttribute] = ...
    kRotatedLorentzConeConstraint: ClassVar[ProgramAttribute] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class ProgramType(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kCGP: ClassVar[ProgramType] = ...
    kGP: ClassVar[ProgramType] = ...
    kLCP: ClassVar[ProgramType] = ...
    kLP: ClassVar[ProgramType] = ...
    kMILP: ClassVar[ProgramType] = ...
    kMIQP: ClassVar[ProgramType] = ...
    kMISDP: ClassVar[ProgramType] = ...
    kMISOCP: ClassVar[ProgramType] = ...
    kNLP: ClassVar[ProgramType] = ...
    kQP: ClassVar[ProgramType] = ...
    kQuadraticCostConicConstraint: ClassVar[ProgramType] = ...
    kSDP: ClassVar[ProgramType] = ...
    kSOCP: ClassVar[ProgramType] = ...
    kUnknown: ClassVar[ProgramType] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class ProjectedGradientDescentSolver(SolverInterface):
    BacktrackingAlpha0OptionName: ClassVar[nanobind.nb_func] = ...
    BacktrackingCOptionName: ClassVar[nanobind.nb_func] = ...
    BacktrackingTauOptionName: ClassVar[nanobind.nb_func] = ...
    ConvergenceTolOptionName: ClassVar[nanobind.nb_func] = ...
    MaxIterationsOptionName: ClassVar[nanobind.nb_func] = ...
    id: ClassVar[nanobind.nb_func] = ...
    kDefaultBacktrackingAlpha0: ClassVar[float] = ...
    kDefaultBacktrackingC: ClassVar[float] = ...
    kDefaultBacktrackingTau: ClassVar[float] = ...
    kDefaultConvergenceTol: ClassVar[float] = ...
    kDefaultMaxIterations: ClassVar[int] = ...
    kDefaultMaxLineSearchSteps: ClassVar[int] = ...
    def __init__(self) -> None: ...
    def SetCustomGradientFunction(self, custom_gradient_function) -> None: ...
    def SetCustomProjectionFunction(self, custom_projection_function) -> None: ...
    def SetProjectionSolverInterface(self, projection_solver_interface: SolverInterface | None) -> None: ...

class PyFunctionConstraint(Constraint):
    def __init__(self, num_vars: int, func: collections.abc.Callable, lb, ub, description: str = ...) -> None: ...
    def UpdateLowerBound(self, new_lb) -> None: ...
    def UpdateUpperBound(self, new_ub) -> None: ...
    def set_bounds(self, lower_bound, upper_bound) -> None: ...

class PyFunctionCost(Cost):
    def __init__(self, num_vars: int, func: collections.abc.Callable, description: str = ...) -> None: ...

class QuadraticConstraint(Constraint):
    class HessianType(enum.Enum):
        __new__: ClassVar[Callable] = ...
        _generate_next_value_: ClassVar[Callable] = ...
        _hashable_values_: ClassVar[list] = ...
        _member_map_: ClassVar[dict] = ...
        _member_names_: ClassVar[list] = ...
        _member_type_: ClassVar[type[object]] = ...
        _unhashable_values_: ClassVar[list] = ...
        _unhashable_values_map_: ClassVar[dict] = ...
        _use_args_: ClassVar[bool] = ...
        _value2member_map_: ClassVar[dict] = ...
        _value_repr_: ClassVar[None] = ...
        kIndefinite: ClassVar[QuadraticConstraint.HessianType] = ...
        kNegativeSemidefinite: ClassVar[QuadraticConstraint.HessianType] = ...
        kPositiveSemidefinite: ClassVar[QuadraticConstraint.HessianType] = ...
        kZero: ClassVar[QuadraticConstraint.HessianType] = ...
        __nb_enum__: ClassVar[PyCapsule] = ...
    def __init__(self, Q0, b, lb: float, ub: float, hessian_type: QuadraticConstraint.HessianType | None = ...) -> None: ...
    def Q(self, *args, **kwargs): ...
    def UpdateCoefficients(self, new_Q, new_b, hessian_type: QuadraticConstraint.HessianType | None = ...) -> None: ...
    def b(self, *args, **kwargs): ...
    def hessian_type(self) -> QuadraticConstraint.HessianType: ...
    def is_convex(self) -> bool: ...

class QuadraticCost(Cost):
    def __init__(self, Q, b, c: float, is_convex: bool | None = ...) -> None: ...
    def Q(self, *args, **kwargs): ...
    def UpdateCoefficients(self, new_Q, new_b, new_c: float = ..., is_convex: bool | None = ...) -> None: ...
    def UpdateHessianEntry(self, i: int, j: int, val: float, is_hessian_psd: bool | None = ...) -> None: ...
    def b(self, *args, **kwargs): ...
    def c(self) -> float: ...
    def is_convex(self) -> bool: ...
    def update_constant_term(self, new_c: float) -> None: ...
    def update_linear_coefficient_entry(self, i: int, val: float) -> None: ...

class RemoveFreeVariableMethod(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kLorentzConeSlack: ClassVar[RemoveFreeVariableMethod] = ...
    kNullspace: ClassVar[RemoveFreeVariableMethod] = ...
    kTwoSlackVariables: ClassVar[RemoveFreeVariableMethod] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...
    def __index__(self) -> int: ...

class RotatedLorentzConeConstraint(Constraint):
    def __init__(self, A, b) -> None: ...
    def A(self) -> scipy.sparse.csc_matrix[float]: ...
    def UpdateCoefficients(self, new_A, new_b) -> None: ...
    def b(self, *args, **kwargs): ...

class ScsSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class ScsSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def dual_objective(self) -> float: ...
    @property
    def duality_gap(self) -> float: ...
    @property
    def iter(self) -> int: ...
    @property
    def primal_objective(self) -> float: ...
    @property
    def primal_residue(self) -> float: ...
    @property
    def residue_infeasibility(self) -> float: ...
    @property
    def residue_unbounded_a(self) -> float: ...
    @property
    def residue_unbounded_p(self) -> float: ...
    @property
    def s(self): ...
    @property
    def scs_setup_time(self) -> float: ...
    @property
    def scs_solve_time(self) -> float: ...
    @property
    def scs_status(self) -> int: ...
    @property
    def y(self): ...

class SemidefiniteRelaxationOptions:
    add_implied_linear_constraints: bool
    add_implied_linear_equality_constraints: bool
    def __init__(self, **kwargs) -> None: ...
    def set_to_strongest(self) -> None: ...
    def set_to_weakest(self) -> None: ...

class SnoptSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class SnoptSolverDetails:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def F(self) -> Any: ...
    @property
    def Fmul(self): ...
    @property
    def info(self): ...
    @property
    def solve_time(self) -> float: ...
    @property
    def xmul(self): ...

class SolutionResult(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kDualInfeasible: ClassVar[SolutionResult] = ...
    kInfeasibleConstraints: ClassVar[SolutionResult] = ...
    kInfeasibleOrUnbounded: ClassVar[SolutionResult] = ...
    kInvalidInput: ClassVar[SolutionResult] = ...
    kIterationLimit: ClassVar[SolutionResult] = ...
    kSolutionFound: ClassVar[SolutionResult] = ...
    kSolutionResultNotSet: ClassVar[SolutionResult] = ...
    kSolverSpecificError: ClassVar[SolutionResult] = ...
    kUnbounded: ClassVar[SolutionResult] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class SolverId:
    def __init__(self, name: str) -> None: ...
    def name(self) -> str: ...
    def __eq__(self, arg: SolverId) -> bool: ...
    def __hash__(self) -> int: ...
    def __ne__(self, arg: SolverId) -> bool: ...

class SolverInterface:
    def __init__(self) -> None: ...
    def AreProgramAttributesSatisfied(self, prog: MathematicalProgram) -> bool: ...
    def ExplainUnsatisfiedProgramAttributes(self, prog: MathematicalProgram) -> str: ...
    @overload
    def Solve(self, prog: MathematicalProgram, initial_guess, solver_options: SolverOptions | None, result: MathematicalProgramResult) -> None: ...
    @overload
    def Solve(self, prog: MathematicalProgram, initial_guess=..., solver_options: SolverOptions | None = ...) -> MathematicalProgramResult: ...
    def SolverName(self) -> str: ...
    def available(self) -> bool: ...
    def enabled(self) -> bool: ...
    def solver_id(self) -> SolverId: ...
    def solver_type(self) -> SolverType | None: ...

class SolverOptions:
    options: Incomplete
    def __init__(self, options: collections.abc.Mapping[str, collections.abc.Mapping[str, float | int | str]] = ...) -> None: ...
    @overload
    def SetOption(self, solver_id: SolverId, key: str, value: float | int | str) -> None: ...
    @overload
    def SetOption(self, key: CommonSolverOption, value: float | int | str) -> None: ...
    def __copy__(self) -> SolverOptions: ...
    def __deepcopy__(self, arg: dict) -> SolverOptions: ...
    def __eq__(self, arg: SolverOptions) -> bool: ...
    def __ne__(self, arg: SolverOptions) -> bool: ...
    @property
    def __fields__(self): ...

class SolverType(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kClp: ClassVar[SolverType] = ...
    kCsdp: ClassVar[SolverType] = ...
    kEqualityConstrainedQP: ClassVar[SolverType] = ...
    kGurobi: ClassVar[SolverType] = ...
    kIpopt: ClassVar[SolverType] = ...
    kLinearSystem: ClassVar[SolverType] = ...
    kMobyLCP: ClassVar[SolverType] = ...
    kMosek: ClassVar[SolverType] = ...
    kNlopt: ClassVar[SolverType] = ...
    kOsqp: ClassVar[SolverType] = ...
    kScs: ClassVar[SolverType] = ...
    kSnopt: ClassVar[SolverType] = ...
    kUnrevisedLemke: ClassVar[SolverType] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class UnrevisedLemkeSolver(SolverInterface):
    id: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...

class VisualizationCallback(EvaluatorBase):
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7solvers7BindingINS0_10ConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, c: Constraint, v) -> None: ...
    @overload
    def __init__(self, arg: object) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> Constraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_10L1NormCostEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: L1NormCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> L1NormCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTL1NormCostU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_10L2NormCostEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: L2NormCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> L2NormCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTL2NormCostU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_10LinearCostEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearCostU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_12LInfNormCostEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LInfNormCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LInfNormCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLInfNormCostU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_13EvaluatorBaseEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, c: EvaluatorBase, v) -> None: ...
    @overload
    def __init__(self, arg: object) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> EvaluatorBase: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTEvaluatorBaseU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_13QuadraticCostEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: QuadraticCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> QuadraticCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTQuadraticCostU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_14ExpressionCostEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: ExpressionCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> ExpressionCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTExpressionCostU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_16LinearConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_19QuadraticConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: QuadraticConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> QuadraticConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTQuadraticConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_20ExpressionConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: ExpressionConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> ExpressionConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTExpressionConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_21BoundingBoxConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: BoundingBoxConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> BoundingBoxConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTBoundingBoxConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_21LorentzConeConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LorentzConeConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LorentzConeConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLorentzConeConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_21VisualizationCallbackEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: VisualizationCallback, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> VisualizationCallback: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTVisualizationCallbackU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_24LinearEqualityConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearEqualityConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearEqualityConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearEqualityConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_24PerspectiveQuadraticCostEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: PerspectiveQuadraticCost, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> PerspectiveQuadraticCost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTPerspectiveQuadraticCostU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_25ExponentialConeConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: ExponentialConeConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> ExponentialConeConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTExponentialConeConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_28RotatedLorentzConeConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: RotatedLorentzConeConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> RotatedLorentzConeConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTRotatedLorentzConeConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_30PositiveSemidefiniteConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: PositiveSemidefiniteConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> PositiveSemidefiniteConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTPositiveSemidefiniteConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_31LinearComplementarityConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearComplementarityConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearComplementarityConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearComplementarityConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_32LinearMatrixInequalityConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: LinearMatrixInequalityConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> LinearMatrixInequalityConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTLinearMatrixInequalityConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_32MinimumValueLowerBoundConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: MinimumValueLowerBoundConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> MinimumValueLowerBoundConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTMinimumValueLowerBoundConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_32MinimumValueUpperBoundConstraintEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, c: MinimumValueUpperBoundConstraint, v) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> MinimumValueUpperBoundConstraint: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTMinimumValueUpperBoundConstraintU) -> bool: ...
    def __hash__(self) -> int: ...

class _TemporaryName_N5drake7solvers7BindingINS0_4CostEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, c: Cost, v) -> None: ...
    @overload
    def __init__(self, arg: object) -> None: ...
    @overload
    def ToLatex(self, precision: int = ...) -> str: ...
    @overload
    def ToLatex(self) -> Any: ...
    def evaluator(self) -> Cost: ...
    def variables(self, *args, **kwargs): ...
    def __eq__(self, arg: BindingTCostU) -> bool: ...
    def __hash__(self) -> int: ...



# See `ExecuteExtraPythonCode` in `pydrake_pybind.h` for usage details and
# rationale.

# ruff: noqa: F821 (undefined-name). This file is only a fragment.

import functools
import operator
import sys
import typing


def logical_and(*formulas):
    assert len(formulas) >= 1, "Must supply at least one operand"
    return functools.reduce(__logical_and, formulas)


def logical_or(*formulas):
    assert len(formulas) >= 1, "Must supply at least one operand"
    return functools.reduce(__logical_or, formulas)


def _reduce_add(*args):
    return functools.reduce(operator.add, args)


def _reduce_mul(*args):
    return functools.reduce(operator.mul, args)


# Drake's SymPy support is loaded lazily (on demand), so that Drake does not
# directly depend on SymPy. The implementation lives in `_symbolic_sympy.py`.
_symbolic_sympy_defer = None


def to_sympy(
    x: float | int | bool | Variable | Expression | Formula,
    *,
    memo: dict | None = None,
) -> typing.Union[float, int, bool, "sympy.Expr"]:
    """Converts a pydrake object to the corresponding SymPy Expr.

    Certain expressions are not supported and will raise NotImplementedError.
    (Most acutely, note that `int` is not yet supported.)

    This function aims to support the latest contemporaneous version of SymPy
    as of Drake's release date.

    Args:
        x: The pydrake object to be converted.
        memo: (Optional) Mapping between Drake variables and SymPy variables.
            Converting a ``pydrake.symbolic.Variable`` to SymPy will look up
            the Drake variable's ``v.get_id()`` as a key in this dictionary.
            If a value is found, it will be used as the SymPy atom for that
            Drake variable.  Otherwise, a new SymPy variable will be created
            and used, and both mappings ``{drake_var.get_id(): sympy_var}``
            and ``{sympy_var: drake_var}`` will be inserted into ``memo``.

    See also :meth:`pydrake.symbolic.from_sympy`.
    """
    global _symbolic_sympy_defer
    if _symbolic_sympy_defer is None:
        from pydrake.symbolic import _symbolic_sympy as _symbolic_sympy_defer
    if memo is None:
        memo = dict()
    return _symbolic_sympy_defer._to_sympy(x, memo=memo)


def from_sympy(
    x: typing.Union[float, int, bool, "sympy.Expr"], *, memo: dict | None = None
) -> float | int | bool | Variable | Expression | Formula:
    """Converts a SymPy Expr to the corresponding pydrake object.

    Certain expressions are not supported and will raise NotImplementedError.

    This function aims to support the latest contemporaneous version of SymPy
    as of Drake's release date.

    Args:
        x: The SymPy object to be converted.
        memo: (Optional) Mapping between SymPy variables and Drake variables.
            Converting a SymPy variable to a ``pydrake.symbolic.Variable`` will
            look up the SymPy variable as a key in this dictionary.
            If a value is found, then it will be used as the Drake variable.
            If no value is found, then (for now) raises an exception. In the
            future, we might support automatically creating variables, but
            that is not yet implemented.

    See also :meth:`pydrake.symbolic.to_sympy`.
    """
    global _symbolic_sympy_defer
    if _symbolic_sympy_defer is None:
        from pydrake.symbolic import _symbolic_sympy as _symbolic_sympy_defer
    if memo is None:
        memo = dict()
    return _symbolic_sympy_defer._from_sympy(x, memo=memo)


# We must be able to do `from pydrake.symbolic import _symbolic_sympy` so we
# need `pydrake.symbolic` to be a Python package, not merely a module. (See
# https://docs.python.org/3/tutorial/modules.html for details.) The way to
# designate something as a package is to define its `__path__` attribute.
__path__ = [sys.modules["pydrake"].__path__[0] + "/symbolic"]



import pydrake.multibody.contact_solvers
import pydrake.systems.analysis
import pydrake.systems.framework
from pydrake.common.cpp_template import CenicIntegrator_ as CenicIntegrator_
from typing import ClassVar

class CenicIntegrator(pydrake.systems.analysis.IntegratorBase):
    def __init__(self, system: pydrake.systems.framework.System, context: pydrake.systems.framework.Context | None = ...) -> None: ...
    def SetSolverParameters(self, parameters: pydrake.multibody.contact_solvers.IcfSolverParameters) -> None: ...
    def get_solver_parameters(self) -> pydrake.multibody.contact_solvers.IcfSolverParameters: ...

class CenicIntegrator_𝓣AutoDiffXd𝓤(pydrake.systems.analysis.IntegratorBase_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, system: pydrake.systems.framework.System_TAutoDiffXdU, context: pydrake.systems.framework.Context_TAutoDiffXdU | None = ...) -> None: ...
    def SetSolverParameters(self, parameters: pydrake.multibody.contact_solvers.IcfSolverParameters) -> None: ...
    def get_solver_parameters(self) -> pydrake.multibody.contact_solvers.IcfSolverParameters: ...

class _TemporaryName_N5drake9multibody15CenicIntegratorINS_2ad8AutoDiffEEE(pydrake.systems.analysis.IntegratorBase_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, system: pydrake.systems.framework.System_TAutoDiffXdU, context: pydrake.systems.framework.Context_TAutoDiffXdU | None = ...) -> None: ...
    def SetSolverParameters(self, parameters: pydrake.multibody.contact_solvers.IcfSolverParameters) -> None: ...
    def get_solver_parameters(self) -> pydrake.multibody.contact_solvers.IcfSolverParameters: ...



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

import drake

class lcmt_hydroelastic_contact_surface_for_viz(object):

    __slots__ = ["geometry1_name", "body1_name", "model1_name", "body1_unique", "collision_count1", "geometry2_name", "body2_name", "model2_name", "body2_unique", "collision_count2", "centroid_W", "force_C_W", "moment_C_W", "num_quadrature_points", "quadrature_point_data", "num_vertices", "p_WV", "pressure", "poly_data_int_count", "poly_data"]

    __typenames__ = ["string", "string", "string", "boolean", "int32_t", "string", "string", "string", "boolean", "int32_t", "double", "double", "double", "int32_t", "drake.lcmt_hydroelastic_quadrature_per_point_data_for_viz", "int32_t", "drake.lcmt_point", "double", "int32_t", "int32_t"]

    __dimensions__ = [None, None, None, None, None, None, None, None, None, None, [3], [3], [3], None, ["num_quadrature_points"], None, ["num_vertices"], ["num_vertices"], None, ["poly_data_int_count"]]

    def __init__(self):
        self.geometry1_name = ""
        """
        The contact is between two bodies, but we track multiple names per body
        so that visualizers can fully disambiguate contacts:
        
        - name of the geometry affixed to the body which produced the surface.
        - name of the body.
        - name of the model instance to which the body belongs.
        - uniqueness of the body name. If unique, the body can unambiguously be
        represented by just its body name. Otherwise it must be combined with
        its model instance name. The validity of this logic relies on
        MultibodyPlant's requirement that model instance names must be unique.
        - the number of collision geometries affixed to the body. If the
        colliding bodies both have a single collision geometry each, there can
        be only one contact surface between them and display can be
        streamlined. Otherwise, the possibility of multiple contact surfaces
        needs to be accounted for.
        
        LCM Type: string
        """

        self.body1_name = ""
        """ LCM Type: string """
        self.model1_name = ""
        """ LCM Type: string """
        self.body1_unique = False
        """ LCM Type: boolean """
        self.collision_count1 = 0
        """ LCM Type: int32_t """
        self.geometry2_name = ""
        """ LCM Type: string """
        self.body2_name = ""
        """ LCM Type: string """
        self.model2_name = ""
        """ LCM Type: string """
        self.body2_unique = False
        """ LCM Type: boolean """
        self.collision_count2 = 0
        """ LCM Type: int32_t """
        self.centroid_W = [ 0.0 for dim0 in range(3) ]
        """
        The centroid of the contact surface, as an offset vector expressed in the
        world frame.
        LCM Type: double[3]
        """

        self.force_C_W = [ 0.0 for dim0 in range(3) ]
        """
        The force, expressed in the world frame, that is applied to `body1_name`
        at the centroid of the contact surface.
        LCM Type: double[3]
        """

        self.moment_C_W = [ 0.0 for dim0 in range(3) ]
        """
        The moment, expressed in the world frame, that is applied to `body1_name`
        at the centroid of the contact surface.
        LCM Type: double[3]
        """

        self.num_quadrature_points = 0
        """
        Vestigial. Always zero.
        LCM Type: int32_t
        """

        self.quadrature_point_data = []
        """
        Vestigial. Always empty.
        LCM Type: drake.lcmt_hydroelastic_quadrature_per_point_data_for_viz[num_quadrature_points]
        """

        self.num_vertices = 0
        """
        The vertices.
        LCM Type: int32_t
        """

        self.p_WV = []
        """
        TODO(SeanCurtis-TRI): It would be nice to have a simple Vector3 type here,
        but the concept of a 3D point is perfectly sufficient.
        LCM Type: drake.lcmt_point[num_vertices]
        """

        self.pressure = []
        """
        Pressure values at each vertex.
        LCM Type: double[num_vertices]
        """

        self.poly_data_int_count = 0
        """
        The polygons are encoded in one long stream as
        c0, v00, v01, ..., c1, v10, v11, ...
        Such that each polygon has a count of vertices ci, followed by ci number
        of 0-indexed indices into the vertex set. The total number of integer
        values is recorded here.
        LCM Type: int32_t
        """

        self.poly_data = []
        """ LCM Type: int32_t[poly_data_int_count] """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_hydroelastic_contact_surface_for_viz._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        __geometry1_name_encoded = self.geometry1_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__geometry1_name_encoded)+1))
        buf.write(__geometry1_name_encoded)
        buf.write(b"\0")
        __body1_name_encoded = self.body1_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__body1_name_encoded)+1))
        buf.write(__body1_name_encoded)
        buf.write(b"\0")
        __model1_name_encoded = self.model1_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__model1_name_encoded)+1))
        buf.write(__model1_name_encoded)
        buf.write(b"\0")
        buf.write(struct.pack(">bi", self.body1_unique, self.collision_count1))
        __geometry2_name_encoded = self.geometry2_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__geometry2_name_encoded)+1))
        buf.write(__geometry2_name_encoded)
        buf.write(b"\0")
        __body2_name_encoded = self.body2_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__body2_name_encoded)+1))
        buf.write(__body2_name_encoded)
        buf.write(b"\0")
        __model2_name_encoded = self.model2_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__model2_name_encoded)+1))
        buf.write(__model2_name_encoded)
        buf.write(b"\0")
        buf.write(struct.pack(">bi", self.body2_unique, self.collision_count2))
        buf.write(struct.pack('>3d', *self.centroid_W[:3]))
        buf.write(struct.pack('>3d', *self.force_C_W[:3]))
        buf.write(struct.pack('>3d', *self.moment_C_W[:3]))
        buf.write(struct.pack(">i", self.num_quadrature_points))
        for i0 in range(self.num_quadrature_points):
            assert self.quadrature_point_data[i0]._get_packed_fingerprint() == drake.lcmt_hydroelastic_quadrature_per_point_data_for_viz._get_packed_fingerprint()
            self.quadrature_point_data[i0]._encode_one(buf)
        buf.write(struct.pack(">i", self.num_vertices))
        for i0 in range(self.num_vertices):
            assert self.p_WV[i0]._get_packed_fingerprint() == drake.lcmt_point._get_packed_fingerprint()
            self.p_WV[i0]._encode_one(buf)
        buf.write(struct.pack('>%dd' % self.num_vertices, *self.pressure[:self.num_vertices]))
        buf.write(struct.pack(">i", self.poly_data_int_count))
        buf.write(struct.pack('>%di' % self.poly_data_int_count, *self.poly_data[:self.poly_data_int_count]))

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_hydroelastic_contact_surface_for_viz._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_hydroelastic_contact_surface_for_viz._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_hydroelastic_contact_surface_for_viz()
        __geometry1_name_len = struct.unpack('>I', buf.read(4))[0]
        self.geometry1_name = buf.read(__geometry1_name_len)[:-1].decode('utf-8', 'replace')
        __body1_name_len = struct.unpack('>I', buf.read(4))[0]
        self.body1_name = buf.read(__body1_name_len)[:-1].decode('utf-8', 'replace')
        __model1_name_len = struct.unpack('>I', buf.read(4))[0]
        self.model1_name = buf.read(__model1_name_len)[:-1].decode('utf-8', 'replace')
        self.body1_unique = bool(struct.unpack('b', buf.read(1))[0])
        self.collision_count1 = struct.unpack(">i", buf.read(4))[0]
        __geometry2_name_len = struct.unpack('>I', buf.read(4))[0]
        self.geometry2_name = buf.read(__geometry2_name_len)[:-1].decode('utf-8', 'replace')
        __body2_name_len = struct.unpack('>I', buf.read(4))[0]
        self.body2_name = buf.read(__body2_name_len)[:-1].decode('utf-8', 'replace')
        __model2_name_len = struct.unpack('>I', buf.read(4))[0]
        self.model2_name = buf.read(__model2_name_len)[:-1].decode('utf-8', 'replace')
        self.body2_unique = bool(struct.unpack('b', buf.read(1))[0])
        self.collision_count2 = struct.unpack(">i", buf.read(4))[0]
        self.centroid_W = struct.unpack('>3d', buf.read(24))
        self.force_C_W = struct.unpack('>3d', buf.read(24))
        self.moment_C_W = struct.unpack('>3d', buf.read(24))
        self.num_quadrature_points = struct.unpack(">i", buf.read(4))[0]
        self.quadrature_point_data = []
        for i0 in range(self.num_quadrature_points):
            self.quadrature_point_data.append(drake.lcmt_hydroelastic_quadrature_per_point_data_for_viz._decode_one(buf))
        self.num_vertices = struct.unpack(">i", buf.read(4))[0]
        self.p_WV = []
        for i0 in range(self.num_vertices):
            self.p_WV.append(drake.lcmt_point._decode_one(buf))
        self.pressure = struct.unpack('>%dd' % self.num_vertices, buf.read(self.num_vertices * 8))
        self.poly_data_int_count = struct.unpack(">i", buf.read(4))[0]
        self.poly_data = struct.unpack('>%di' % self.poly_data_int_count, buf.read(self.poly_data_int_count * 4))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_hydroelastic_contact_surface_for_viz in parents: return 0
        newparents = parents + [lcmt_hydroelastic_contact_surface_for_viz]
        tmphash = (0x4efd7d8f630a0b66+ drake.lcmt_hydroelastic_quadrature_per_point_data_for_viz._get_hash_recursive(newparents)+ drake.lcmt_point._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_hydroelastic_contact_surface_for_viz._packed_fingerprint is None:
            lcmt_hydroelastic_contact_surface_for_viz._packed_fingerprint = struct.pack(">Q", lcmt_hydroelastic_contact_surface_for_viz._get_hash_recursive([]))
        return lcmt_hydroelastic_contact_surface_for_viz._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_hydroelastic_contact_surface_for_viz._get_packed_fingerprint())[0]




import nanobind
from typing import Any, Callable, overload

ExtractGradient: nanobind.nb_func
ExtractValue: nanobind.nb_func
InitializeAutoDiff: nanobind.nb_func
InitializeAutoDiffTuple: Callable

class AutoDiffXd:
    @overload
    def __init__(self, value: float) -> None: ...
    @overload
    def __init__(self, value: float, derivatives) -> None: ...
    @overload
    def __init__(self, value: float, size: int, offset: int) -> None: ...
    def abs(self) -> AutoDiffXd: ...
    def acos(self) -> AutoDiffXd: ...
    def arccos(self, *args, **kwargs): ...
    def arcsin(self, *args, **kwargs): ...
    def arctan(self) -> AutoDiffXd: ...
    def arctan2(self, *args, **kwargs): ...
    def asin(self) -> AutoDiffXd: ...
    def atan(self) -> AutoDiffXd: ...
    @overload
    def atan2(self, x: AutoDiffXd) -> AutoDiffXd: ...
    @overload
    def atan2(self, y, x) -> Any: ...
    def ceil(self) -> float: ...
    def cos(self) -> AutoDiffXd: ...
    def cosh(self) -> AutoDiffXd: ...
    def derivatives(self, *args, **kwargs): ...
    def exp(self) -> AutoDiffXd: ...
    def floor(self) -> float: ...
    def log(self) -> AutoDiffXd: ...
    def max(self, arg: AutoDiffXd) -> AutoDiffXd: ...
    def min(self, arg: AutoDiffXd) -> AutoDiffXd: ...
    def pow(self, arg: float) -> AutoDiffXd: ...
    def sin(self) -> AutoDiffXd: ...
    def sinh(self) -> AutoDiffXd: ...
    def sqrt(self) -> AutoDiffXd: ...
    def tan(self) -> AutoDiffXd: ...
    def tanh(self) -> AutoDiffXd: ...
    def value(self) -> float: ...
    @overload
    def __abs__(self) -> AutoDiffXd: ...
    @overload
    def __abs__(self) -> AutoDiffXd: ...
    def __add__(self, arg: AutoDiffXd) -> AutoDiffXd: ...
    def __ceil__(self) -> float: ...
    def __copy__(self) -> AutoDiffXd: ...
    def __deepcopy__(self, arg: dict) -> AutoDiffXd: ...
    def __eq__(self, arg: AutoDiffXd) -> bool: ...
    def __floor__(self) -> float: ...
    def __ge__(self, arg: AutoDiffXd) -> bool: ...
    def __gt__(self, arg: AutoDiffXd) -> bool: ...
    def __le__(self, arg: AutoDiffXd) -> bool: ...
    def __lt__(self, arg: AutoDiffXd) -> bool: ...
    def __mul__(self, arg: AutoDiffXd) -> AutoDiffXd: ...
    def __ne__(self, arg: AutoDiffXd) -> bool: ...
    def __neg__(self) -> AutoDiffXd: ...
    def __pow__(self, arg: float) -> AutoDiffXd: ...
    def __radd__(self, arg: float) -> AutoDiffXd: ...
    def __rmul__(self, arg: float) -> AutoDiffXd: ...
    def __rsub__(self, arg: float) -> AutoDiffXd: ...
    def __rtruediv__(self, arg: float) -> AutoDiffXd: ...
    def __sub__(self, arg: AutoDiffXd) -> AutoDiffXd: ...
    def __truediv__(self, arg: AutoDiffXd) -> AutoDiffXd: ...



"""Contains routines to monkey patch or provide alternatives to upstream code
which may need changes for compatibility.
"""

import numpy as np

_patches = {
    "numpy_formatters": {
        "applied": False,
        "required": np.lib.NumpyVersion(np.__version__) < "1.13.0",
    },
}


def _defer_callable_type(cls):
    # Makes a type, which is meant to purely callable, and defers its
    # construction until it needs to be called.

    class Deferred:
        def __init__(self, *args, **kwargs):
            self._args = args
            self._kwargs = kwargs
            self._obj = None

        def _get_obj(self):
            if self._obj is None:
                self._obj = cls(*self._args, **self._kwargs)
            return self._obj

        def __call__(self, *args, **kwargs):
            return self._get_obj().__call__(*args, **kwargs)

    return Deferred


def maybe_patch_numpy_formatters():
    """Required to permit printing of symbolic array types in NumPy < 1.13.0.

    See #8729 for more information.
    """
    # Provides version-dependent monkey-patch which effectively achieves a
    # portion of https://github.com/numpy/numpy/pull/8963
    patch = _patches["numpy_formatters"]
    if not patch["required"]:
        return
    if patch["applied"]:
        return
    module = np.core.arrayprint
    defer_callable_types = [
        "IntegerFormat",
        "FloatFormat",
    ]
    for name in defer_callable_types:
        original = getattr(module, name)
        deferred = _defer_callable_type(original)
        setattr(module, name, deferred)
    patch["applied"] = True



import collections.abc
import nanobind
import os
import pydrake.common.schema
import pydrake.geometry
import pydrake.multibody.plant
import pydrake.multibody.tree
import pydrake.systems.framework
from _typeshed import Incomplete
from typing import ClassVar, overload

FlattenModelDirectives: nanobind.nb_func
GetScopedFrameByName: nanobind.nb_func
GetScopedFrameByNameMaybe: nanobind.nb_func
LoadModelDirectives: nanobind.nb_func
LoadModelDirectivesFromString: nanobind.nb_func
ProcessModelDirectives: nanobind.nb_func

class AddCollisionFilterGroup:
    ignored_collision_filter_groups: list[str]
    member_groups: list[str]
    members: list[str]
    model_namespace: str | None
    name: str
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> AddCollisionFilterGroup: ...
    def __deepcopy__(self, arg: dict) -> AddCollisionFilterGroup: ...
    @property
    def __fields__(self): ...

class AddDirectives:
    file: str
    model_namespace: str | None
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> AddDirectives: ...
    def __deepcopy__(self, arg: dict) -> AddDirectives: ...
    @property
    def __fields__(self): ...

class AddFrame:
    X_PF: pydrake.common.schema.Transform
    name: str
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> AddFrame: ...
    def __deepcopy__(self, arg: dict) -> AddFrame: ...
    @property
    def __fields__(self): ...

class AddModel:
    default_free_body_pose: dict[str, pydrake.common.schema.Transform]
    default_joint_positions: Incomplete
    file: str
    name: str
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> AddModel: ...
    def __deepcopy__(self, arg: dict) -> AddModel: ...
    @property
    def __fields__(self): ...

class AddModelInstance:
    name: str
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> AddModelInstance: ...
    def __deepcopy__(self, arg: dict) -> AddModelInstance: ...
    @property
    def __fields__(self): ...

class AddWeld:
    X_PC: pydrake.common.schema.Transform | None
    child: str
    parent: str
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> AddWeld: ...
    def __deepcopy__(self, arg: dict) -> AddWeld: ...
    @property
    def __fields__(self): ...

class CollisionFilterGroups:
    def __init__(self) -> None: ...
    def AddExclusionPair(self, pair: tuple[str, str]) -> None: ...
    def AddGroup(self, name: str, members: collections.abc.Set[str]) -> None: ...
    def empty(self) -> bool: ...
    def exclusion_pairs(self) -> set[tuple[str, str]]: ...
    def groups(self) -> dict[str, set[str]]: ...

class ModelDirective:
    add_collision_filter_group: Incomplete
    add_directives: Incomplete
    add_frame: Incomplete
    add_model: Incomplete
    add_model_instance: Incomplete
    add_weld: Incomplete
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> ModelDirective: ...
    def __deepcopy__(self, arg: dict) -> ModelDirective: ...
    @property
    def __fields__(self): ...

class ModelDirectives:
    directives: Incomplete
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> ModelDirectives: ...
    def __deepcopy__(self, arg: dict) -> ModelDirectives: ...
    @property
    def __fields__(self): ...

class ModelInstanceInfo:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def X_PC(self): ...
    @property
    def child_frame_name(self) -> str: ...
    @property
    def model_instance(self): ...
    @property
    def model_name(self) -> str: ...
    @property
    def model_path(self) -> str: ...
    @property
    def parent_frame_name(self) -> str: ...

class PackageMap:
    class RemoteParams:
        archive_type: Incomplete
        sha256: str
        strip_prefix: Incomplete
        urls: list[str]
        def __init__(self, **kwargs) -> None: ...
        def ToJson(self) -> str: ...
        def __copy__(self) -> PackageMap.RemoteParams: ...
        def __deepcopy__(self, arg: dict) -> PackageMap.RemoteParams: ...
        @property
        def __fields__(self): ...
    MakeEmpty: ClassVar[nanobind.nb_func] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, other: PackageMap) -> None: ...
    def Add(self, package_name: str, package_path: str | os.PathLike) -> None: ...
    def AddMap(self, other_map: PackageMap) -> None: ...
    def AddPackageXml(self, filename: str | os.PathLike) -> None: ...
    def AddRemote(self, package_name: str, params: PackageMap.RemoteParams) -> None: ...
    def Contains(self, package_name: str) -> bool: ...
    def GetPackageNames(self) -> list[str]: ...
    def GetPath(self, package_name: str) -> str: ...
    def PopulateFromEnvironment(self, environment_variable: str) -> None: ...
    def PopulateFromFolder(self, path: str | os.PathLike) -> None: ...
    def PopulateFromRosPackagePath(self) -> None: ...
    def Remove(self, package_name: str) -> None: ...
    def ResolveUrl(self, url: str) -> str: ...
    def size(self) -> int: ...
    def __copy__(self) -> PackageMap: ...
    def __deepcopy__(self, arg: dict) -> PackageMap: ...

class Parser:
    @overload
    def __init__(self, plant: pydrake.multibody.plant.MultibodyPlant, scene_graph: pydrake.geometry.SceneGraph | None = ..., model_name_prefix: str = ...) -> None: ...
    @overload
    def __init__(self, plant: pydrake.multibody.plant.MultibodyPlant, model_name_prefix: str) -> None: ...
    @overload
    def __init__(self, builder: pydrake.systems.framework.DiagramBuilder | None, plant: pydrake.multibody.plant.MultibodyPlant | None = ..., scene_graph: pydrake.geometry.SceneGraph | None = ..., model_name_prefix: str = ...) -> None: ...
    @overload
    def AddModels(self, file_name: str | os.PathLike) -> list[pydrake.multibody.tree.ModelInstanceIndex]: ...
    @overload
    def AddModels(self, url: str) -> list[pydrake.multibody.tree.ModelInstanceIndex]: ...
    def AddModelsFromString(self, file_contents: str, file_type: str) -> list[pydrake.multibody.tree.ModelInstanceIndex]: ...
    def AddModelsFromUrl(self, url: str) -> list[pydrake.multibody.tree.ModelInstanceIndex]: ...
    def GetAutoRenaming(self) -> bool: ...
    def GetCollisionFilterGroups(self) -> CollisionFilterGroups: ...
    def SetAutoRenaming(self, value: bool) -> None: ...
    def SetStrictParsing(self) -> None: ...
    def builder(self) -> pydrake.systems.framework.DiagramBuilder: ...
    def package_map(self) -> PackageMap: ...
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph: ...
